// Program mierzy czas utrzymywania sie stanu wysokiego na linii 0 portu A
// ("szerokosc" impulsu - czas pomiedzy kolejnymi zboczami: rosnacym i opadajacym)
// wykorzystujac tryb Input Caputre ukladu licznikowego (opis - rozdzial 18.3.5).
//
// Zadania obowiazkowe:
// 1. Ustalic rozdzielczosc i zakres pomiaru, dokonczyc konfiguracje timera TIM2
// 2. Sprawdzic poprawnosc wynikow porownujac:
//
// - czas nacisniecia przycisku ze wskazaniem stopera/zegarka,
// - szerokosc impulsu pochodzacego z generatora wzorcowego
//   (z nastawami generatora, ew. wskazaniami miernika czestotliwosci / oscyloskopu).
//
// 3. Zadanie dodatkowe - dopisac obsluge przekroczenia zakresu pomiarowego.

#include <stdio.h>
#include "stm32f4xx.h"
#include "GLCD.h"

#define __FI       1
  #define __FONT_WIDTH  16
  #define __FONT_HEIGHT 24

char tekst[20];

int main(void){ 
	
// --- Inicjalizacja i konfiguracja wyswietlacza LCD ---
	
	GLCD_Initialize(); 
	GLCD_Clear(White);
	GLCD_SetBackColor(White);
  GLCD_SetTextColor(Black);

// --- Konfiguracja peryferiow ---
	
  RCC->AHB1ENR |= ((1<<0) | (1<<6));       		// Wlacz "zegar" dla GPIO A i G.
	RCC->APB1ENR |= (1<<0);   									// Wlacz "zegar" dla TIM2.

  GPIOG->MODER |= (1<<(2*13)) | (1<<(2*14));	// Ustaw linie G13 i G14 jako wyjscia.
	
	GPIOA->MODER |= (2<<2*0);										// Linia 0 portu A - ALTERNATE FUNCTION AF1 TIM2 CH1
	GPIOA->AFR[0] |= (1<<4*0);

// --- 1a --- 
// Ustal zakres pomiarowy i rozdzielczosc pomiaru.

	TIM2->PSC	=			 													// Preskaler.
	TIM2->ARR =																// Opcjonalnie zakres zliczania (0...n).

// --- 1b --- 
// Konfiguracja timera TIM2 w trybie Input Capture (kanal 1). Ustawic bity:
// CC1S = 01 - Capture Compare ch. 1 jako wejscie,
// sygnal IC1 pobierany z linii T1,
// Rozdzial 18.3.4 - schemat nr 159.
// Opis rejestru w rozdziale 18.4.7 (Reference Manual).

	TIM2->CCMR1 =

// Sterowanie licznikiem (Slave Mode / SMS). Ustawic bity:
// SMS = 100 - Slave Reset Mode - tryb zerowania (reset) w chwili nadejscia zbocza rosnacego,
// TS = 101 - Trigger Selection - Timer Input 1 - wejscie synchronizacji pracy licznika.
// Opis rejestru w rozdziale 18.4.3 (Reference Manual).
	
	TIM2->SMCR =

// Wlaczyc tryb pracy "Capture" kanalu 1. - zapis wartosci chwilowej licznika CNT do rejestru CCR1. Ustawic bity:
// CC1E - Capture Enabled,
// CC1P i CC1NP - reakcja wejscia sterujacego praca timera na oba zbocza: rosnace i opadajace.
// Opis rejestru w rozdziale 18.4.9 (Reference Manual).

	TIM2->CCER =
	
// --- 3a --- Obsluga przekroczenia zakresu pomiarowego.

//	TIM2->DIER =												// UIE - wlacz przerwanie od przepelnienia (Update) TIM2.
//	TIM2->CR1  =												// Update Request Source: 1 - tylko od przepelnienia
																				// 0 - rowniez w chwili przechwytywania i zerowania.
																				// Rozdzialy 18.4.1 / 18.4. Reference Manual.

// Wlacz linie przerwan: zewnetrzne EXTI0 i timera TIM7 w kontrolerze przerwan (NVIC).
// Tablica wektorow przerwan - Table 63. Reference Manual.

//	NVIC->ISER[0] =

	TIM2->EGR = 1;												// Ponowna inicjalizacja licznika - po zmianie konfiguracji.
  TIM2->CR1 |= 1;												// Wlacz licznik.

// -----------------

	for(;;) {
		
// Program glowny: 		
// Wyswietl odpowiedni komunikat gdy trwa pomiar, a po jego zaknczeniu - zmierzony czas.

// Czas nalezy odczytac (ew. wyskalowac) z rejestru Caputer/Compare kanalu 1 timera 2 (CCR1).

// --- 3c --- Mozna zmodyfikowac o programowa obsluge przekroczenia zakresu pomiarowego.

			if (GPIOA->IDR & 1) GLCD_DisplayString(1, 0, __FI,"Pomiar!             ");
			else {
				sprintf(tekst,"Czas = %u ms      ",TIM2->CCR1);
				GLCD_DisplayString(1, 0, __FI,tekst);
			}
	}
}


// --- 3b --- Przekroczenie zakresu pomiarowego:
// procedura obslugi przerwania przepelnienia TIM2.

/*
void TIM2_IRQHandler(void) 
{	
			TIM2->SR &= ~1;
}
*/
